//          Copyright Jason Wagner 2008
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

#ifndef BOOST_LOGGING_CONDITIONS_HPP_
#define BOOST_LOGGING_CONDITIONS_HPP_
#include <boost/mpl/at.hpp>
#include <boost/mpl/logical.hpp>
#include <boost/mpl/apply.hpp>
#include <boost/mpl/comparison.hpp>

namespace boost { namespace logging { namespace condition {

/**
 * Conditions are used for compile-time evaluation of whether to log information.
 * Names are in (equality "or" list), not_in (inequality "or" list), between, greater_equal,
 * less_equal, greater, and less.
 */

template<class Value1,                  class Value2=boost::logging::details::null_tag,class Value3=boost::logging::details::null_tag,class Value4=boost::logging::details::null_tag,class Value5=boost::logging::details::null_tag,
         class Value6=boost::logging::details::null_tag,class Value7=boost::logging::details::null_tag,class Value8=boost::logging::details::null_tag,class Value9=boost::logging::details::null_tag,class Value10=boost::logging::details::null_tag>
struct in {
  template<class TagMap>
  struct apply
  {
   template<class Param>
    struct subclause {
      typedef typename boost::mpl::at<TagMap,typename Param::category>::type base;
      typedef typename boost::mpl::and_<
                           typename boost::mpl::not_<typename boost::is_same<Param,boost::logging::details::null_tag>::type>::type,
                           typename boost::is_same<typename Param::value,base>::type
                       >::type type;
    };

    typedef typename
      boost::mpl::or_<
        boost::mpl::or_<
          typename subclause<Value1>::type,
          typename subclause<Value2>::type,
          typename subclause<Value3>::type,
          typename subclause<Value4>::type,
          typename subclause<Value5>::type
        >,
        boost::mpl::or_<
          typename subclause<Value6>::type,
          typename subclause<Value7>::type,
          typename subclause<Value8>::type,
          typename subclause<Value9>::type,
          typename subclause<Value10>::type
        >
      >::type
    type;
  };
};


template<class Value1,                  class Value2=boost::logging::details::null_tag,class Value3=boost::logging::details::null_tag,class Value4=boost::logging::details::null_tag,class Value5=boost::logging::details::null_tag,
         class Value6=boost::logging::details::null_tag,class Value7=boost::logging::details::null_tag,class Value8=boost::logging::details::null_tag,class Value9=boost::logging::details::null_tag,class Value10=boost::logging::details::null_tag>
struct not_in {
  template<class TagMap>
  struct apply
  {
    typedef typename boost::mpl::not_<
      typename boost::mpl::apply<
        in<Value1,Value2,Value3,Value4,Value5,Value6,Value7,Value8,Value9,Value10>,
        TagMap
      >::type
    >::type type;
  };
};

template<class MinTag, class MaxTag>
struct between
{
  BOOST_MPL_ASSERT((boost::is_same<typename MinTag::category,typename MaxTag::category>));
  BOOST_MPL_ASSERT((boost::mpl::less_equal<typename MinTag::value,typename MaxTag::value>));

  template<class TagMap>
  struct apply
  {
    typedef typename boost::mpl::if_<
      typename boost::is_same<typename boost::mpl::at<TagMap,typename MinTag::category>::type,boost::mpl::void_>,
      boost::logging::details::null_tag,
      typename boost::mpl::at<TagMap,typename MinTag::category>::type
    >::type val;

    typedef typename boost::mpl::and_<
          typename boost::mpl::not_<typename boost::is_same<val,boost::logging::details::null_tag>::type>::type,
          boost::mpl::less_equal<val,typename MaxTag::value>,
          boost::mpl::greater_equal<val,typename MinTag::value>
    >::type type;
  };
};

template<class ReferenceTag>
struct greater_equal
{
  template<class TagMap>
  struct apply
  {
    typedef typename boost::mpl::if_<
      typename boost::is_same<typename boost::mpl::at<TagMap,typename ReferenceTag::category>::type,boost::mpl::void_>,
      boost::logging::details::null_tag,
      typename boost::mpl::at<TagMap,typename ReferenceTag::category>::type
    >::type val;

    typedef typename boost::mpl::and_<
          typename boost::mpl::not_<typename boost::is_same<val,boost::logging::details::null_tag>::type>::type,
          boost::mpl::greater_equal<val,typename ReferenceTag::value>
    >::type type;
  };
};

template<class ReferenceTag>
struct less_equal
{
  template<class TagMap>
  struct apply
  {
    typedef typename boost::mpl::if_<
      typename boost::is_same<typename boost::mpl::at<TagMap,typename ReferenceTag::category>::type,boost::mpl::void_>,
      boost::logging::details::null_tag,
      typename boost::mpl::at<TagMap,typename ReferenceTag::category>::type
    >::type val;

    typedef typename boost::mpl::and_<
          typename boost::mpl::not_<typename boost::is_same<val,boost::logging::details::null_tag>::type>::type,
          boost::mpl::less_equal<val,typename ReferenceTag::value>
    >::type type;
  };
};

template<class ReferenceTag>
struct greater
{
  template<class TagMap>
  struct apply
  {
    typedef typename boost::mpl::if_<
      typename boost::is_same<typename boost::mpl::at<TagMap,typename ReferenceTag::category>::type,boost::mpl::void_>,
      boost::logging::details::null_tag,
      typename boost::mpl::at<TagMap,typename ReferenceTag::category>::type
    >::type val;

    typedef typename boost::mpl::and_<
          typename boost::mpl::not_<typename boost::is_same<val,boost::logging::details::null_tag>::type>::type,
          boost::mpl::greater<val,typename ReferenceTag::value>
    >::type type;
  };
};
template<class ReferenceTag>
struct less
{
  template<class TagMap>
  struct apply
  {
    typedef typename boost::mpl::if_<
      typename boost::is_same<typename boost::mpl::at<TagMap,typename ReferenceTag::category>::type,boost::mpl::void_>,
      boost::logging::details::null_tag,
      typename boost::mpl::at<TagMap,typename ReferenceTag::category>::type
    >::type val;

    typedef typename boost::mpl::and_<
          typename boost::mpl::not_<typename boost::is_same<val,boost::logging::details::null_tag>::type>::type,
          boost::mpl::less<val,typename ReferenceTag::value>
    >::type type;
  };
};
}}} // namespace boost::logging::condition

#endif /*BOOST_LOGGING_CONDITIONS_HPP_*/
