//          Copyright Jason Wagner 2008
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

#ifndef BOOST_LOGGING_DETAILS_MESSAGE_HELPER_HPP_
#define BOOST_LOGGING_DETAILS_MESSAGE_HELPER_HPP_

#include <boost/mpl/map.hpp>
#include <boost/mpl/insert.hpp>
#include <boost/mpl/logical.hpp>
#include <boost/mpl/fold.hpp>
#include <boost/type_traits/integral_constant.hpp>

#include <boost/logging/details/message_fwd.hpp>

namespace boost { namespace logging { namespace details {

struct tag_vector_to_map {
  template<class TagMap,class NewTag>
  struct apply
  {
    // if you get this assertion, you've put two tags from the same category in the default tag list
    BOOST_MPL_ASSERT((boost::mpl::not_<typename boost::mpl::has_key<TagMap,typename NewTag::category>::type>));
    typedef typename boost::mpl::insert<TagMap,
                                    boost::mpl::begin<TagMap>,
                                    boost::mpl::pair<typename NewTag::category,typename NewTag::value>
                                   >::type type;
  };
};

template<class TagMap>
struct message_eval_impl
{
  template<class Value,class Condition>
  struct apply {
    typedef typename boost::mpl::and_<Value,boost::mpl::apply<Condition,TagMap> >::type type;
  };
};


template<class TagConditions, class TagMap>
struct message_eval
{
    typedef typename boost::mpl::fold<TagConditions, boost::mpl::true_ , message_eval_impl<TagMap> >::type type;
};

template<class Category, int Value, class TagMap, class LoggerType, class IsRecordOpen>
struct message_return {
  typedef typename boost::mpl::insert<TagMap,
                                      boost::mpl::begin<TagMap>,
                                      boost::mpl::pair<Category,boost::mpl::int_<Value> >
                                     >::type NewTagMap;
  typedef message<typename message_eval<typename LoggerType::ConditionMap,NewTagMap>::type,
        NewTagMap, LoggerType,IsRecordOpen > type;
};

}}} //namespace boost::log::details
#endif /*BOOST_LOGGING_DETAILS_MESSAGE_HELPER_HPP_*/
