//          Copyright Jason Wagner 2008
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

#ifndef BOOST_LOGGING_DETAILS_VARIABLE_COMPARISIONS_HPP_
#define BOOST_LOGGING_DETAILS_VARIABLE_COMPARISIONS_HPP_

#include <typeinfo>
namespace boost { namespace logging { namespace details {

// The typeid(L).before(typeid(R)) != 0 is there because MSVC complains about converting
// an int to a bool.  Ok, thanks for that MS.

struct equals :public boost::static_visitor<bool> {
  template<class L, class R>
  bool operator()(const L& l, const R& r) const
  {
    return false;
  }
  template<class L>
  bool operator()(const L& l, const L& r) const
  {
    return l==r;
  }
};
struct not_equals :public boost::static_visitor<bool> {
  template<class L, class R>
  bool operator()(const L& l, const R& r) const
  {
    return true;
  }
  template<class L>
  bool operator()(const L& l, const L& r) const
  {
    return l!=r;
  }
};

struct less_than :public boost::static_visitor<bool> {

  template<class L, class R>
  bool operator()(const L& l, const R& r) const
  {
    return typeid(L).before(typeid(R))!=0;
  }
  template<class L>
  bool operator()(const L& l, const L& r) const
  {
    return l<r;
  }
};

struct less_equal :public boost::static_visitor<bool> {
  template<class L, class R>
  bool operator()(const L& l, const R& r) const
  { // if they are equal, this function won't be called
    return typeid(L).before(typeid(R))!=0;
  }
  template<class L>
  bool operator()(const L& l, const L& r) const
  {
    return l<=r;
  }
};
struct greater_than :public boost::static_visitor<bool> {
  template<class L, class R>
  bool operator()(const L& l, const R& r) const
  {
    return typeid(R).before(typeid(L))!=0;
  }
  template<class L>
  bool operator()(const L& l, const L& r) const
  {
    return l>r;
  }
};
struct greater_equal :public boost::static_visitor<bool> {
  template<class L, class R>
  bool operator()(const L& l, const R& r) const
  {
    return typeid(R).before(typeid(L))!=0;
  }
  template<class L>
  bool operator()(const L& l, const L& r) const
  {
    return l>=r;
  }
};
}}} // namespace boost::logging::details

#endif //BOOST_LOGGING_DETAILS_VARIABLE_COMPARISIONS_HPP_
