//          Copyright Jason Wagner 2008
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

#ifndef BOOST_LOGGING_MANIPULATORS_HPP_
#define BOOST_LOGGING_MANIPULATORS_HPP_

namespace boost { namespace logging {


/**
 * WARNING: I don't like this at all.  Not even sure if it's correct.  I'll probably convert it to ostream style
 * manipulator if I get a chance.
 *
 * The trick to this is that template overloads occur before non-templates, and our <<(TYPE t) overload eats
 * everything, so it needs to be a more specific template overload to accept a manipulator.
 * Note: Or it can be ambiguous with a non-template overload that disambiguates,
 *       but I figured that out after I wrote this code.
 * The <<(Manipulator<>) function is templated on the Op, so it gets called over the less specific TYPE.
 *
 * The problem I have is that the implementation does:
 * @code
 * template<class Op>
 * return_type operator<<(const manipulator<Op>& t)
 * {
 *   static_cast<const Op&>(t)(*this,true);
 * }
 * @endcode
 * This casts to the baseclass and calls the templated function which is instantiated based upon
 * *this.  The mix of templates and static_casts with inheritance worries me, though I can't figure out any
 * definite problem with it.
 *
 * It's also ugly to define new manipulators and you can't define templated manipulators.
 */
template<class Op>
struct manipulator : public Op
{
  template<class T1,class T2, class T3, class T4, class T5>
  manipulator(T1 t1, T2 t2,T3 t3,T4 t4, T5 t5) : Op(t1,t2,t3,t4,t5) {}

  template<class T1,class T2, class T3, class T4>
  manipulator(T1 t1, T2 t2,T3 t3,T4 t4) : Op(t1,t2,t3,t4) {}

  template<class T1,class T2, class T3>
  manipulator(T1 t1, T2 t2,T3 t3) : Op(t1,t2,t3) {}

  template<class T1,class T2>
  manipulator(T1 t1, T2 t2) : Op(t1,t2) {}

  template<class T1>
  manipulator(T1 t1) : Op(t1) {}

  manipulator() {}
};

struct end_record_impl {
  template<class Logger>
  void operator()(Logger& l,bool /*is_active*/) const
  {
    l.end_record();
  }
};
typedef manipulator<end_record_impl> end_record;

struct eol_impl  {
  template<class Logger>
  void operator()(Logger& l,bool is_active) const
  {
    if(is_active)
      l.write("\n");
  }
};
typedef manipulator<eol_impl> eol;


}} // namespace boost::logging
#endif /*BOOST_LOGGING_MANIPULATORS_HPP_*/
