//          Copyright Jason Wagner 2008
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

#ifndef BOOST_LOGGING_ST_VARSET_HPP_
#define BOOST_LOGGING_ST_VARSET_HPP_
#include <map>

#include <boost/logging/variable.hpp>

namespace boost { namespace logging {
struct single_threaded_variable_set : public variable_set {
  const value_type& get(const std::string& vt) const
  {
    // check the local context
    variable_map::const_iterator var=variables_.find(vt);
    if(var != variables_.end())
      return var->second;
    if(parent_)
      return parent_->get(vt);
    return null_variable;
  }

  virtual bool set(const std::string& vt,const value_type& val)
  {
    variables_[vt]=val;
    return false;
  }
  virtual void clear()
  {
    variables_.clear();
  }

protected:
  typedef std::map<std::string, value_type> variable_map;
  variable_map variables_;

};
}}//namespace boost::logging
#endif // BOOST_LOGGING_ST_VARSET_HPP_
