//          Copyright Jason Wagner 2008
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

#ifndef BOOST_LOGGING_TAG_HPP_
#define BOOST_LOGGING_TAG_HPP_

#include <string>
#include<boost/mpl/int.hpp>
#include<boost/mpl/void.hpp>
namespace boost { namespace logging {
template<class C, int V>
struct tag
{
  typedef C category;
  typedef boost::mpl::int_<V> value ;
};

namespace details {
  struct null_tag {
    typedef boost::mpl::void_ category;
    typedef boost::mpl::int_<0> value;
  };
};

//=============================================
// Categories -- severity

struct severity
{
  typedef boost::logging::tag<severity, 900> emergency;
  typedef boost::logging::tag<severity, 800> alert;
  typedef boost::logging::tag<severity, 700> critical;
  typedef boost::logging::tag<severity, 600> error;
  typedef boost::logging::tag<severity, 500> warning;
  typedef boost::logging::tag<severity, 400> notice;
  typedef boost::logging::tag<severity, 300> info;
  typedef boost::logging::tag<severity, 200> debug;
  typedef boost::logging::tag<severity, 100> trace;
  const char* type_string() const
  {
    return "severity";
  }

  const char* toString(int i) const {
    switch(i) {
      case 100: return "EMERGENCY";
      case 200: return "ALERT";
      case 300: return "CRITICAL";
      case 400: return "ERROR";
      case 500: return "WARNING";
      case 600: return "NOTICE";
      case 700: return "INFO";
      case 800: return "DEBUG";
      case 900: return "TRACE";
      default:  return "UNKNOWN SEVERITY";
    }
  };
};

//=============================================
// Categories -- build_target
struct build_target
{
  typedef boost::logging::tag<build_target, 1> exe;
  typedef boost::logging::tag<build_target, 2> static_lib;
  typedef boost::logging::tag<build_target, 3> dynamic_lib;
  const char* type_string() const {
    return "build_target";
  }
  const char* toString(int i) const {
    switch(i) {
      case 1: return "EXECUTABLE";
      case 2: return "STATIC LIBRARY";
      case 3: return "DYNAMIC LIBRARY";
      default: return "UNKNOWN BUILD TARGET";
    }
  };
};

//=============================================
// Categories -- build_type
struct build_type
{
  typedef boost::logging::tag<build_type, 100> debug;
  typedef boost::logging::tag<build_type, 200> release;
  const char* type_string() const {
    return "build_type";
  }
  const char* toString(int i)const  {
    switch(i) {
      case 100: return "DEBUG";
      case 200: return "RELEASE";
      default: return "UNKNOWN BUILD TYPE";
    }
  };
};

}} // namespace boost::logging
#endif /*BOOST_LOG_TAG_HPP_*/
