//          Copyright Jason Wagner 2008
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

#ifndef BOOST_LOGGING_VARIABLE_HPP_
#define BOOST_LOGGING_VARIABLE_HPP_
#include <boost/variant.hpp>
#include <boost/shared_ptr.hpp>
namespace boost{namespace logging{

typedef boost::variant<std::string, int, double> default_variable_type;

struct variable
{
  typedef default_variable_type value_type;
  variable(const value_type& v) : val(v) { }
  variable() : val() {}

  const value_type& get() const
  {
    return val;
  }
  virtual const char* type_string() const =0;

protected:
  value_type val;
};

const default_variable_type null_variable;

struct variable_set
{
  typedef default_variable_type value_type;
  typedef boost::shared_ptr<variable_set> ptr;

  virtual const value_type& get(const std::string& vt) const=0;
  virtual bool set(const std::string& vt,const value_type&) =0;
  virtual void clear()=0;
  bool set(const variable& vt)
  { return this->set(vt.type_string(),vt.get());}

  const value_type& get(const variable& vt) const
  {  return this->get(vt.type_string()); }

  void set_parent(variable_set::ptr p)
  { parent_=p;}

protected:
  variable_set::ptr parent_;
};

// TODO clean this up, not fully thought out
struct file_name: public variable
{
  const char* type_string() const
  {
    return "file_name";
  }
};
}
}//namespace boost::logging
#endif /*BOOST_LOGGING_VARIABLE_HPP_*/
