//          Copyright Jason Wagner 2008
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

#ifndef BOOST_LOGGING_LAZY_HPP_
#define BOOST_LOGGING_LAZY_HPP_

#include <boost/bind.hpp>
#include <boost/function.hpp>

namespace boost { namespace logging {

/** A set of functions that builds a boost::function holding a boost::bind expression.
 * This is useful for expensive functions that we don't want to evaluate if the result
 * is not going to be logged.
 * TODO more overloads for more arguments.  This is just a proof of concept
 */

template <class RV, class T1, class A1>
boost::function<RV ()> lazy(RV (*f)(T1),A1 a1)
{
  return boost::function<RV ()>(boost::bind(f,a1));
}

template <class RV, class Class, class T1, class A1>
boost::function<RV ()> lazy(RV (Class::*f)(T1),Class* c,A1 a1)
{
  return boost::function<RV ()>(boost::bind(f,c,a1));
}

}} //namespace boost::logging

#endif // BOOST_LOGGING_LAZY_HPP_
